@@ -14,6 +14,7 @@ use crate::format::Format;
 use crate::format::IncompatiblePixelsType;
 use crate::format::Pixel;
 use crate::image::ImageAccess;
+use crate::image::ImageDimensions;
 use crate::image::SampleCount;
 use crate::DeviceSize;
 use crate::VulkanObject;
@@ -106,6 +107,32 @@ where
         return Err(CheckCopyBufferImageError::ImageCoordinatesOutOfRange);
     }
 
+    match image.dimensions() {
+        ImageDimensions::Dim1d { .. } => {
+            // VUID-vkCmdCopyBufferToImage-srcImage-00199
+            if image_offset[1] != 0 || image_size[1] != 1 {
+                return Err(CheckCopyBufferImageError::ImageCoordinatesOutOfRange);
+            }
+
+            // VUID-vkCmdCopyBufferToImage-srcImage-00201
+            if image_offset[2] != 0 || image_size[2] != 1 {
+                return Err(CheckCopyBufferImageError::ImageCoordinatesOutOfRange);
+            }
+        }
+        ImageDimensions::Dim2d { .. } => {
+            // VUID-vkCmdCopyBufferToImage-srcImage-00201
+            if image_offset[2] != 0 || image_size[2] != 1 {
+                return Err(CheckCopyBufferImageError::ImageCoordinatesOutOfRange);
+            }
+        }
+        ImageDimensions::Dim3d { .. } => {
+            // VUID-vkCmdCopyBufferToImage-baseArrayLayer-00213
+            if image_first_layer != 0 || image_num_layers != 1 {
+                return Err(CheckCopyBufferImageError::ImageCoordinatesOutOfRange);
+            }
+        }
+    }
+
     Px::ensure_accepts(image.format())?;
 
     {
@@ -134,7 +161,7 @@ fn required_len_for_format<Px>(
 where
     Px: Pixel,
 {
-    let (block_width, block_height) = format.block_dimensions();
+    let [block_width, block_height] = format.block_dimensions();
     let num_blocks = (image_size[0] + block_width - 1) / block_width
         * ((image_size[1] + block_height - 1) / block_height)
         * image_size[2]
@@ -153,40 +180,40 @@ mod tests {
     fn test_required_len_for_format() {
         // issue #1292
         assert_eq!(
-            required_len_for_format::<u8>(Format::BC1_RGBUnormBlock, [2048, 2048, 1], 1),
+            required_len_for_format::<u8>(Format::BC1_RGB_UNORM_BLOCK, [2048, 2048, 1], 1),
             2097152
         );
         // other test cases
         assert_eq!(
-            required_len_for_format::<u8>(Format::R8G8B8A8Unorm, [2048, 2048, 1], 1),
+            required_len_for_format::<u8>(Format::R8G8B8A8_UNORM, [2048, 2048, 1], 1),
             16777216
         );
         assert_eq!(
-            required_len_for_format::<u8>(Format::R4G4UnormPack8, [512, 512, 1], 1),
+            required_len_for_format::<u8>(Format::R4G4_UNORM_PACK8, [512, 512, 1], 1),
             262144
         );
         assert_eq!(
-            required_len_for_format::<u8>(Format::R8G8B8Uscaled, [512, 512, 1], 1),
+            required_len_for_format::<u8>(Format::R8G8B8_USCALED, [512, 512, 1], 1),
             786432
         );
         assert_eq!(
-            required_len_for_format::<u8>(Format::R32G32Uint, [512, 512, 1], 1),
+            required_len_for_format::<u8>(Format::R32G32_UINT, [512, 512, 1], 1),
             2097152
         );
         assert_eq!(
-            required_len_for_format::<u32>(Format::R32G32Uint, [512, 512, 1], 1),
+            required_len_for_format::<u32>(Format::R32G32_UINT, [512, 512, 1], 1),
             524288
         );
         assert_eq!(
-            required_len_for_format::<[u32; 2]>(Format::R32G32Uint, [512, 512, 1], 1),
+            required_len_for_format::<[u32; 2]>(Format::R32G32_UINT, [512, 512, 1], 1),
             262144
         );
         assert_eq!(
-            required_len_for_format::<u8>(Format::ASTC_8x8UnormBlock, [512, 512, 1], 1),
+            required_len_for_format::<u8>(Format::ASTC_8x8_UNORM_BLOCK, [512, 512, 1], 1),
             65536
         );
         assert_eq!(
-            required_len_for_format::<u8>(Format::ASTC_12x12SrgbBlock, [512, 512, 1], 1),
+            required_len_for_format::<u8>(Format::ASTC_12x12_SRGB_BLOCK, [512, 512, 1], 1),
             29584
         );
     }
