@@ -9,9 +9,8 @@
 
 use crate::check_errors;
 use crate::descriptor_set::layout::DescriptorDesc;
-use crate::descriptor_set::layout::DescriptorDescTy;
-use crate::descriptor_set::layout::DescriptorSetCompatibilityError;
 use crate::descriptor_set::layout::DescriptorSetDesc;
+use crate::descriptor_set::layout::DescriptorType;
 use crate::descriptor_set::pool::DescriptorsCount;
 use crate::device::Device;
 use crate::device::DeviceOwned;
@@ -31,10 +30,8 @@ pub struct DescriptorSetLayout {
     device: Arc<Device>,
     // Descriptors.
     desc: DescriptorSetDesc,
-    // Number of descriptors.
+    // Number of descriptors of each type.
     descriptors_count: DescriptorsCount,
-    // Number of descriptors in a variable count descriptor. Will be zero if no variable count descriptors are present.
-    variable_descriptor_count: u32,
 }
 
 impl DescriptorSetLayout {
@@ -45,21 +42,37 @@ impl DescriptorSetLayout {
     /// empty, you can make the iterator yield `None` for an element.
     pub fn new<D>(
         device: Arc<Device>,
-        desc: D,
-    ) -> Result<DescriptorSetLayout, DescriptorSetLayoutError>
+        set_desc: D,
+    ) -> Result<Arc<DescriptorSetLayout>, DescriptorSetLayoutError>
     where
         D: Into<DescriptorSetDesc>,
     {
-        let desc = desc.into();
+        let set_desc = set_desc.into();
         let mut descriptors_count = DescriptorsCount::zero();
-        let mut variable_descriptor_count = 0;
-        let bindings = desc.bindings();
+        let bindings = set_desc.bindings();
         let mut bindings_vk = Vec::with_capacity(bindings.len());
         let mut binding_flags_vk = Vec::with_capacity(bindings.len());
         let mut immutable_samplers_vk: Vec<Box<[ash::vk::Sampler]>> = Vec::new(); // only to keep the arrays of handles alive
 
-        for (binding, desc) in bindings.iter().enumerate() {
-            let desc = match desc {
+        let mut flags = ash::vk::DescriptorSetLayoutCreateFlags::empty();
+
+        if set_desc.is_push_descriptor() {
+            if !device.enabled_extensions().khr_push_descriptor {
+                return Err(DescriptorSetLayoutError::ExtensionNotEnabled {
+                    extension: "khr_push_descriptor",
+                    reason: "description was set to be a push descriptor",
+                });
+            }
+
+            // TODO: VUID-VkDescriptorSetLayoutCreateInfo-flags-04590
+            // If flags contains VK_DESCRIPTOR_SET_LAYOUT_CREATE_PUSH_DESCRIPTOR_BIT_KHR, flags must
+            // not contain VK_DESCRIPTOR_SET_LAYOUT_CREATE_HOST_ONLY_POOL_BIT_VALVE
+
+            flags |= ash::vk::DescriptorSetLayoutCreateFlags::PUSH_DESCRIPTOR_KHR;
+        }
+
+        for (binding, binding_desc) in bindings.iter().enumerate() {
+            let binding_desc = match binding_desc {
                 Some(d) => d,
                 None => continue,
             };
@@ -67,16 +80,53 @@ impl DescriptorSetLayout {
             // FIXME: it is not legal to pass eg. the TESSELLATION_SHADER bit when the device
             //        doesn't have tess shaders enabled
 
-            let ty = desc.ty.ty();
-            descriptors_count.add_num(ty, desc.descriptor_count);
-            let mut binding_flags = ash::vk::DescriptorBindingFlags::empty();
-            let immutable_samplers = desc.ty.immutable_samplers();
+            let ty = binding_desc.ty;
 
-            let p_immutable_samplers = if !immutable_samplers.is_empty() {
-                if desc.descriptor_count != immutable_samplers.len() as u32 {
+            if set_desc.is_push_descriptor() {
+                if matches!(
+                    ty,
+                    DescriptorType::StorageBufferDynamic | DescriptorType::UniformBufferDynamic
+                ) {
+                    return Err(DescriptorSetLayoutError::PushDescriptorDynamicBuffer);
+                }
+
+                if binding_desc.variable_count {
+                    return Err(DescriptorSetLayoutError::PushDescriptorVariableCount);
+                }
+
+                // TODO: VUID-VkDescriptorSetLayoutBindingFlagsCreateInfo-flags-03003
+                // If VkDescriptorSetLayoutCreateInfo::flags includes
+                // VK_DESCRIPTOR_SET_LAYOUT_CREATE_PUSH_DESCRIPTOR_BIT_KHR,
+                // then all elements of pBindingFlags must not include
+                // VK_DESCRIPTOR_BINDING_UPDATE_AFTER_BIND_BIT,
+                // VK_DESCRIPTOR_BINDING_UPDATE_UNUSED_WHILE_PENDING_BIT, or
+                // VK_DESCRIPTOR_BINDING_VARIABLE_DESCRIPTOR_COUNT_BIT
+
+                // TODO: VUID-VkDescriptorSetLayoutCreateInfo-flags-02208
+                // If flags contains VK_DESCRIPTOR_SET_LAYOUT_CREATE_PUSH_DESCRIPTOR_BIT_KHR, then all
+                // elements of pBindings must not have a descriptorType of
+                // VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK_EXT
+
+                // TODO: VUID-VkDescriptorSetLayoutCreateInfo-flags-04591
+                // If flags contains VK_DESCRIPTOR_SET_LAYOUT_CREATE_PUSH_DESCRIPTOR_BIT_KHR,
+                // pBindings must not have a descriptorType of VK_DESCRIPTOR_TYPE_MUTABLE_VALVE
+            }
+
+            descriptors_count.add_num(ty, binding_desc.descriptor_count);
+            let mut binding_flags = ash::vk::DescriptorBindingFlags::empty();
+
+            let p_immutable_samplers = if !binding_desc.immutable_samplers.is_empty() {
+                if !matches!(
+                    ty,
+                    DescriptorType::Sampler | DescriptorType::CombinedImageSampler
+                ) {
+                    return Err(DescriptorSetLayoutError::ImmutableSamplersWrongDescriptorType);
+                }
+
+                if binding_desc.descriptor_count != binding_desc.immutable_samplers.len() as u32 {
                     return Err(DescriptorSetLayoutError::ImmutableSamplersCountMismatch {
-                        descriptor_count: desc.descriptor_count,
-                        sampler_count: immutable_samplers.len() as u32,
+                        descriptor_count: binding_desc.descriptor_count,
+                        sampler_count: binding_desc.immutable_samplers.len() as u32,
                     });
                 }
 
@@ -85,7 +135,8 @@ impl DescriptorSetLayout {
                 // with one of the values VK_BORDER_COLOR_FLOAT_CUSTOM_EXT or
                 // VK_BORDER_COLOR_INT_CUSTOM_EXT
 
-                let sampler_handles = immutable_samplers
+                let sampler_handles = binding_desc
+                    .immutable_samplers
                     .iter()
                     .map(|s| s.internal_object())
                     .collect::<Vec<_>>()
@@ -97,65 +148,71 @@ impl DescriptorSetLayout {
                 ptr::null()
             };
 
-            if desc.variable_count {
+            if binding_desc.variable_count {
                 if binding != bindings.len() - 1 {
                     return Err(DescriptorSetLayoutError::VariableCountDescMustBeLast);
                 }
 
-                if desc.ty == DescriptorDescTy::UniformBufferDynamic
-                    || desc.ty == DescriptorDescTy::StorageBufferDynamic
+                if binding_desc.ty == DescriptorType::UniformBufferDynamic
+                    || binding_desc.ty == DescriptorType::StorageBufferDynamic
                 {
                     return Err(DescriptorSetLayoutError::VariableCountDescMustNotBeDynamic);
                 }
 
                 if !device.enabled_features().runtime_descriptor_array {
-                    return Err(DescriptorSetLayoutError::VariableCountIncompatibleDevice(
-                        IncompatibleDevice::MissingFeature(MissingFeature::RuntimeDescriptorArray),
-                    ));
+                    return Err(DescriptorSetLayoutError::FeatureNotEnabled {
+                        feature: "runtime_descriptor_array",
+                        reason: "binding has a variable count",
+                    });
                 }
 
                 if !device
                     .enabled_features()
                     .descriptor_binding_variable_descriptor_count
                 {
-                    return Err(DescriptorSetLayoutError::VariableCountIncompatibleDevice(
-                        IncompatibleDevice::MissingFeature(
-                            MissingFeature::DescriptorBindingVariableDescriptorCount,
-                        ),
-                    ));
+                    return Err(DescriptorSetLayoutError::FeatureNotEnabled {
+                        feature: "descriptor_binding_variable_descriptor_count",
+                        reason: "binding has a variable count",
+                    });
                 }
 
-                if !device.enabled_features().descriptor_binding_partially_bound {
-                    return Err(DescriptorSetLayoutError::VariableCountIncompatibleDevice(
-                        IncompatibleDevice::MissingFeature(
-                            MissingFeature::DescriptorBindingPartiallyBound,
-                        ),
-                    ));
-                }
-
-                variable_descriptor_count = desc.descriptor_count;
-                // TODO: should these be settable separately by the user?
                 binding_flags |= ash::vk::DescriptorBindingFlags::VARIABLE_DESCRIPTOR_COUNT;
-                binding_flags |= ash::vk::DescriptorBindingFlags::PARTIALLY_BOUND;
             }
 
             bindings_vk.push(ash::vk::DescriptorSetLayoutBinding {
                 binding: binding as u32,
                 descriptor_type: ty.into(),
-                descriptor_count: desc.descriptor_count,
-                stage_flags: desc.stages.into(),
+                descriptor_count: binding_desc.descriptor_count,
+                stage_flags: binding_desc.stages.into(),
                 p_immutable_samplers,
             });
             binding_flags_vk.push(binding_flags);
         }
 
-        // Note that it seems legal to have no descriptor at all in the set.
+        if set_desc.is_push_descriptor()
+            && descriptors_count.total()
+                > device
+                    .physical_device()
+                    .properties()
+                    .max_push_descriptors
+                    .unwrap_or(0)
+        {
+            return Err(DescriptorSetLayoutError::MaxPushDescriptorsExceeded {
+                max: device
+                    .physical_device()
+                    .properties()
+                    .max_push_descriptors
+                    .unwrap_or(0),
+                obtained: descriptors_count.total(),
+            });
+        }
 
         let handle = unsafe {
             let binding_flags_infos = if device.api_version() >= Version::V1_2
                 || device.enabled_extensions().ext_descriptor_indexing
             {
                 Some(ash::vk::DescriptorSetLayoutBindingFlagsCreateInfo {
+                    // Note that it seems legal to have no descriptor at all in the set.
                     binding_count: binding_flags_vk.len() as u32,
                     p_binding_flags: binding_flags_vk.as_ptr(),
                     ..Default::default()
@@ -165,7 +222,7 @@ impl DescriptorSetLayout {
             };
 
             let infos = ash::vk::DescriptorSetLayoutCreateInfo {
-                flags: ash::vk::DescriptorSetLayoutCreateFlags::empty(),
+                flags,
                 binding_count: bindings_vk.len() as u32,
                 p_bindings: bindings_vk.as_ptr(),
                 p_next: if let Some(next) = binding_flags_infos.as_ref() {
@@ -190,13 +247,12 @@ impl DescriptorSetLayout {
             output.assume_init()
         };
 
-        Ok(DescriptorSetLayout {
+        Ok(Arc::new(DescriptorSetLayout {
             handle,
             device,
-            desc,
+            desc: set_desc,
             descriptors_count,
-            variable_descriptor_count,
-        })
+        }))
     }
 
     pub(crate) fn desc(&self) -> &DescriptorSetDesc {
@@ -209,10 +265,22 @@ impl DescriptorSetLayout {
         &self.descriptors_count
     }
 
-    /// Returns the number of descriptors in a variable count descriptor. This will return zero if there are no variable count descriptors present.
+    /// If the last binding has a variable count, returns its `descriptor_count`. Otherwise returns
+    /// 0.
     #[inline]
     pub fn variable_descriptor_count(&self) -> u32 {
-        self.variable_descriptor_count
+        self.desc
+            .bindings()
+            .last()
+            .and_then(|binding| binding.as_ref())
+            .map(|binding| {
+                if binding.variable_count {
+                    binding.descriptor_count
+                } else {
+                    0
+                }
+            })
+            .unwrap_or(0)
     }
 
     /// Returns the number of binding slots in the set.
@@ -240,19 +308,6 @@ impl DescriptorSetLayout {
     pub fn is_compatible_with(&self, other: &DescriptorSetLayout) -> bool {
         self.handle == other.handle || self.desc.is_compatible_with(&other.desc)
     }
-
-    /// Checks whether the descriptor of a pipeline layout `self` is compatible with the descriptor
-    /// of a descriptor set being bound `other`.
-    pub fn ensure_compatible_with_bind(
-        &self,
-        other: &DescriptorSetLayout,
-    ) -> Result<(), DescriptorSetCompatibilityError> {
-        if self.handle == other.handle {
-            return Ok(());
-        }
-
-        self.desc.ensure_compatible_with_bind(&other.desc)
-    }
 }
 
 unsafe impl DeviceOwned for DescriptorSetLayout {
@@ -285,11 +340,17 @@ impl Drop for DescriptorSetLayout {
     }
 }
 
-/// Error related to descriptor set layout
+/// Error related to descriptor set layout.
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum DescriptorSetLayoutError {
-    /// Out of Memory
-    OomError(OomError),
+    ExtensionNotEnabled {
+        extension: &'static str,
+        reason: &'static str,
+    },
+    FeatureNotEnabled {
+        feature: &'static str,
+        reason: &'static str,
+    },
 
     /// The number of immutable samplers does not match the descriptor count.
     ImmutableSamplersCountMismatch {
@@ -297,42 +358,32 @@ pub enum DescriptorSetLayoutError {
         sampler_count: u32,
     },
 
-    /// Variable count descriptor must be last binding
+    /// Immutable samplers were included on a descriptor type other than `Sampler` or
+    /// `CombinedImageSampler`.
+    ImmutableSamplersWrongDescriptorType,
+
+    /// The maximum number of push descriptors has been exceeded.
+    MaxPushDescriptorsExceeded {
+        /// Maximum allowed value.
+        max: u32,
+        /// Value that was passed.
+        obtained: u32,
+    },
+
+    /// Out of Memory.
+    OomError(OomError),
+
+    /// The layout was being created for push descriptors, but included a dynamic buffer binding.
+    PushDescriptorDynamicBuffer,
+
+    /// The layout was being created for push descriptors, but included a variable count binding.
+    PushDescriptorVariableCount,
+
+    /// Variable count descriptor must be last binding.
     VariableCountDescMustBeLast,
 
-    /// Variable count descriptor must not be a dynamic buffer
+    /// Variable count descriptor must not be a dynamic buffer.
     VariableCountDescMustNotBeDynamic,
-
-    /// Device is not compatible with variable count descriptors
-    VariableCountIncompatibleDevice(IncompatibleDevice),
-}
-
-// Part of the DescriptorSetLayoutError for the case
-// of missing features on a device.
-#[derive(Debug, Clone, PartialEq, Eq)]
-pub enum IncompatibleDevice {
-    MissingExtension(MissingExtension),
-    MissingFeature(MissingFeature),
-    InsufficientApiVersion {
-        required: Version,
-        obtained: Version,
-    },
-}
-
-// Part of the IncompatibleDevice for the case
-// of missing features on a device.
-#[derive(Debug, Clone, PartialEq, Eq)]
-pub enum MissingFeature {
-    RuntimeDescriptorArray,
-    DescriptorBindingVariableDescriptorCount,
-    DescriptorBindingPartiallyBound,
-}
-
-// Part of the IncompatibleDevice for the case
-// of missing extensions on a device.
-#[derive(Debug, Clone, PartialEq, Eq)]
-pub enum MissingExtension {
-    DescriptorIndexing,
 }
 
 impl From<OomError> for DescriptorSetLayoutError {
@@ -346,32 +397,65 @@ impl std::error::Error for DescriptorSetLayoutError {}
 impl std::fmt::Display for DescriptorSetLayoutError {
     #[inline]
     fn fmt(&self, fmt: &mut std::fmt::Formatter) -> Result<(), std::fmt::Error> {
-        write!(
-            fmt,
-            "{}",
-            match *self {
-                Self::OomError(_) => "out of memory",
-                Self::ImmutableSamplersCountMismatch { .. } =>
-                    "the number of immutable samplers does not match the descriptor count",
-                Self::VariableCountDescMustBeLast =>
-                    "variable count descriptor must be last binding",
-                Self::VariableCountDescMustNotBeDynamic =>
-                    "variable count descriptor must not be a dynamic buffer",
-                Self::VariableCountIncompatibleDevice(_) =>
-                    "device is not compatible with variable count descriptors",
+        match *self {
+            Self::ExtensionNotEnabled { extension, reason } => {
+                write!(
+                    fmt,
+                    "the extension {} must be enabled: {}",
+                    extension, reason
+                )
             }
-        )
+            Self::FeatureNotEnabled { feature, reason } => {
+                write!(fmt, "the feature {} must be enabled: {}", feature, reason)
+            }
+            Self::ImmutableSamplersCountMismatch { .. } => {
+                write!(
+                    fmt,
+                    "the number of immutable samplers does not match the descriptor count"
+                )
+            }
+            Self::ImmutableSamplersWrongDescriptorType => {
+                write!(
+                    fmt,
+                    "immutable samplers were included on a descriptor type other than Sampler or CombinedImageSampler"
+                )
+            }
+            Self::MaxPushDescriptorsExceeded { .. } => {
+                write!(
+                    fmt,
+                    "the maximum number of push descriptors has been exceeded"
+                )
+            }
+            Self::PushDescriptorDynamicBuffer => {
+                write!(fmt, "the layout was being created for push descriptors, but included a dynamic buffer binding")
+            }
+            Self::PushDescriptorVariableCount => {
+                write!(fmt, "the layout was being created for push descriptors, but included a variable count binding")
+            }
+            Self::OomError(_) => {
+                write!(fmt, "out of memory")
+            }
+            Self::VariableCountDescMustBeLast => {
+                write!(fmt, "variable count descriptor must be last binding")
+            }
+            Self::VariableCountDescMustNotBeDynamic => {
+                write!(
+                    fmt,
+                    "variable count descriptor must not be a dynamic buffer"
+                )
+            }
+        }
     }
 }
 
 #[cfg(test)]
 mod tests {
     use crate::descriptor_set::layout::DescriptorDesc;
-    use crate::descriptor_set::layout::DescriptorDescTy;
     use crate::descriptor_set::layout::DescriptorSetDesc;
     use crate::descriptor_set::layout::DescriptorSetLayout;
+    use crate::descriptor_set::layout::DescriptorType;
     use crate::descriptor_set::pool::DescriptorsCount;
-    use crate::pipeline::shader::ShaderStages;
+    use crate::shader::ShaderStages;
     use std::iter;
 
     #[test]
@@ -385,11 +469,11 @@ mod tests {
         let (device, _) = gfx_dev_and_queue!();
 
         let layout = DescriptorDesc {
-            ty: DescriptorDescTy::UniformBuffer,
+            ty: DescriptorType::UniformBuffer,
             descriptor_count: 1,
-            stages: ShaderStages::all_graphics(),
-            mutable: false,
             variable_count: false,
+            stages: ShaderStages::all_graphics(),
+            immutable_samplers: Vec::new(),
         };
 
         let sl = DescriptorSetLayout::new(
