@@ -94,10 +94,10 @@ pub(crate) use self::features::FeaturesFfi;
 pub use self::features::{FeatureRestriction, FeatureRestrictionError, Features};
 pub use self::properties::Properties;
 pub(crate) use self::properties::PropertiesFfi;
-pub use crate::autogen::DeviceExtensions;
 use crate::check_errors;
 use crate::command_buffer::pool::StandardCommandPool;
 use crate::descriptor_set::pool::StdDescriptorPool;
+pub use crate::device::extensions::DeviceExtensions;
 use crate::device::physical::PhysicalDevice;
 use crate::device::physical::QueueFamily;
 pub use crate::extensions::{
@@ -191,8 +191,12 @@ impl Device {
     ///
     /// - Panics if one of the queue families doesn't belong to the given device.
     ///
+
     // TODO: return Arc<Queue> and handle synchronization in the Queue
-    // TODO: should take the PhysicalDevice by value
+
+    // TODO: Eliminate QueuesIter in favour of `impl ExactSizeIterator`. This doesn't currently work
+    // due to this Rust bug: https://github.com/rust-lang/rust/issues/42940. The compiler will
+    // erroneously assume that the return iterator borrows from 'a and break.
     pub fn new<'a, I>(
         physical_device: PhysicalDevice,
         requested_features: &Features,
@@ -401,8 +405,8 @@ impl Device {
     /// Returns the Vulkan version supported by the device.
     ///
     /// This is the lower of the
-    /// [physical device's supported version](crate::instance::PhysicalDevice::api_version) and
-    /// the instance's [`max_api_version`](crate::instance::Instance::max_api_version).
+    /// [physical device's supported version](crate::device::physical::PhysicalDevice::api_version)
+    /// and the instance's [`max_api_version`](crate::instance::Instance::max_api_version).
     #[inline]
     pub fn api_version(&self) -> Version {
         self.api_version
@@ -447,15 +451,11 @@ impl Device {
     /// > **Note**: Will return `-> impl ExactSizeIterator<Item = QueueFamily>` in the future.
     // TODO: ^
     #[inline]
-    pub fn active_queue_families<'a>(
-        &'a self,
-    ) -> Box<dyn ExactSizeIterator<Item = QueueFamily<'a>> + 'a> {
+    pub fn active_queue_families<'a>(&'a self) -> impl ExactSizeIterator<Item = QueueFamily<'a>> {
         let physical_device = self.physical_device();
-        Box::new(
-            self.active_queue_families
-                .iter()
-                .map(move |&id| physical_device.queue_family_by_id(id).unwrap()),
-        )
+        self.active_queue_families
+            .iter()
+            .map(move |&id| physical_device.queue_family_by_id(id).unwrap())
     }
 
     /// Returns the features that have been enabled on the device.
