@@ -541,11 +541,14 @@ impl AttachmentImage {
                 array_layers,
             };
 
-            UnsafeImage::new(
+            UnsafeImage::new_with_exportable_fd(
                 device.clone(),
                 usage,
                 format,
-                ImageCreateFlags::none(),
+                ImageCreateFlags {
+                    mutable_format: true,
+                    ..ImageCreateFlags::none()
+                },
                 dims,
                 samples,
                 1,
@@ -618,15 +621,6 @@ impl AttachmentImage {
     }
 }
 
-impl<A> AttachmentImage<A> {
-    /// Returns the width, height and layers of the image.
-    #[inline]
-    pub fn dimensions(&self) -> [u32; 3] {
-        let dims = self.image.dimensions();
-        [dims.width(), dims.height(), dims.array_layers()]
-    }
-}
-
 unsafe impl<A> ImageAccess for AttachmentImage<A>
 where
     A: MemoryPoolAlloc,
@@ -655,7 +649,7 @@ where
     #[inline]
     fn descriptor_layouts(&self) -> Option<ImageDescriptorLayouts> {
         Some(ImageDescriptorLayouts {
-            storage_image: ImageLayout::ShaderReadOnlyOptimal,
+            storage_image: ImageLayout::General,
             combined_image_sampler: ImageLayout::ShaderReadOnlyOptimal,
             sampled_image: ImageLayout::ShaderReadOnlyOptimal,
             input_attachment: ImageLayout::ShaderReadOnlyOptimal,
@@ -742,11 +736,11 @@ where
 
     #[inline]
     fn current_layer_levels_access(&self) -> std::ops::Range<u32> {
-        0..self.dimensions()[2]
+        0..self.dimensions().array_layers()
     }
 }
 
-unsafe impl<A> ImageClearValue<ClearValue> for Arc<AttachmentImage<A>>
+unsafe impl<A> ImageClearValue<ClearValue> for AttachmentImage<A>
 where
     A: MemoryPoolAlloc,
 {
@@ -756,7 +750,7 @@ where
     }
 }
 
-unsafe impl<P, A> ImageContent<P> for Arc<AttachmentImage<A>>
+unsafe impl<P, A> ImageContent<P> for AttachmentImage<A>
 where
     A: MemoryPoolAlloc,
 {
