@@ -11,7 +11,7 @@ use crate::format::ClearValue;
 use crate::format::Format;
 use crate::image::ImageLayout;
 use crate::image::SampleCount;
-use crate::pipeline::shader::ShaderInterface;
+use crate::shader::ShaderInterface;
 use crate::sync::AccessFlags;
 use crate::sync::PipelineStages;
 
@@ -126,7 +126,10 @@ impl RenderPassDesc {
         };
 
         for element in shader_interface.elements() {
-            for location in element.location.clone() {
+            assert!(!element.ty.is_64bit); // TODO: implement
+            let location_range = element.location..element.location + element.ty.num_locations();
+
+            for location in location_range {
                 let attachment_id = match pass_descr.color_attachments.get(location as usize) {
                     Some(a) => a.0,
                     None => return false,
