@@ -113,7 +113,7 @@
 //!  - How to perform the cycling between images in regard to vsync.
 //!
 //! You can query the supported values of all these properties with
-//! [`Surface::capabilities()]`](struct.Surface.html#method.capabilities).
+//! [`Surface::capabilities`](crate::swapchain::Surface::capabilities).
 //!
 //! ## Creating a swapchain
 //!
@@ -129,7 +129,7 @@
 //! ```
 //!
 //! Then, query the capabilities of the surface with
-//! [`Surface::capabilities()`](struct.Surface.html#method.capabilities)
+//! [`Surface::capabilities`](crate::swapchain::Surface::capabilities)
 //! and choose which values you are going to use.
 //!
 //! ```no_run
@@ -158,7 +158,7 @@
 //! # }
 //! ```
 //!
-//! Then, call [`Swapchain::new()`](struct.Swapchain.html#method.new).
+//! Then, call [`Swapchain::start`](crate::swapchain::Swapchain::start).
 //!
 //! ```no_run
 //! # use std::sync::Arc;
@@ -233,7 +233,7 @@
 //! # let mut swapchain: ::std::sync::Arc<swapchain::Swapchain<()>> = return;
 //! // let mut (swapchain, images) = Swapchain::new(...);
 //! loop {
-//!     # let mut command_buffer: ::vulkano::command_buffer::PrimaryAutoCommandBuffer<()> = return;
+//!     # let mut command_buffer: ::vulkano::command_buffer::PrimaryAutoCommandBuffer = return;
 //!     let (image_num, suboptimal, acquire_future)
 //!         = swapchain::acquire_next_image(swapchain.clone(), None).unwrap();
 //!
@@ -296,18 +296,13 @@
 //! ```
 //!
 
-use std::sync::atomic::AtomicBool;
-
 pub use self::capabilities::Capabilities;
 pub use self::capabilities::ColorSpace;
 pub use self::capabilities::CompositeAlpha;
 pub use self::capabilities::PresentMode;
 pub use self::capabilities::SupportedCompositeAlpha;
-pub use self::capabilities::SupportedCompositeAlphaIter;
 pub use self::capabilities::SupportedPresentModes;
-pub use self::capabilities::SupportedPresentModesIter;
 pub use self::capabilities::SupportedSurfaceTransforms;
-pub use self::capabilities::SupportedSurfaceTransformsIter;
 pub use self::capabilities::SurfaceTransform;
 pub use self::present_region::PresentRegion;
 pub use self::present_region::RectangleLayer;
@@ -327,6 +322,7 @@ pub use self::swapchain::Swapchain;
 pub use self::swapchain::SwapchainAcquireFuture;
 pub use self::swapchain::SwapchainBuilder;
 pub use self::swapchain::SwapchainCreationError;
+use std::sync::atomic::AtomicBool;
 
 mod capabilities;
 pub mod display;
